﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.IO;
using System.Net;
using System.Text.RegularExpressions;
using Ionic.Zip;
using FuzzyStrings;
using MTGRenamer;
using System.Reflection;

namespace MTGRenamer
{
    public class Card
    {
        #region Properties
        private string _AltArt;
        public string AltArt
        {
            get
            {
                return string.IsNullOrEmpty(_AltArt) ? string.Empty : _AltArt;
            }
            set
            {
                _AltArt = value;
            }
        }

        private string _CardText;
        public string CardText
        {
            get
            {
                return string.IsNullOrEmpty(_CardText) ? string.Empty : _CardText;
            }
            set
            {
                _CardText = value;
            }
        }
        public string CardNumber { get; set; }

        private string _Type;
        public string Type
        {
            get
            {
                return string.IsNullOrEmpty(_Type) ? string.Empty : _Type;
            }
            set
            {
                string[] sub = value.Split('-');
                _Type = sub[0].Trim();
                _SubType = sub.Length > 1 ? sub[1].Trim() : string.Empty;
            }
        }

        private string _SubType;
        public string SubType
        {
            get
            {
                return string.IsNullOrEmpty(_SubType) ? string.Empty : _SubType;
            }
            set
            {
                _SubType = value;
            }
        }

        public string Rarity { get; set; }
        public string ID { get; set; }
        private string _Name;
        public virtual string Name
        {
            get
            {
                return _Name;
            }
            set
            {
                _Name = value.Replace("Æ", "Ae");
            }
        }

        private string _AltName;
        public string AltName
        {
            get
            {
                if (_AltName == null)
                {
                    _AltName = String.Format("{0} {1}", this.Name, string.IsNullOrEmpty(this.AltArt) ? string.Empty : " [" + this.AltArt + "]");
                }
                return _AltName;
            }
        }

        public Set Set { get; set; }
        public Card OriginalCard { get; set; }
        public bool IsAltered { get; set; }

        public bool IsInDB
        {
            get
            {
                return Database.Cards.Exists(x => x == this);
            }
        }

        private List<Card> _NeededTokens;
        public List<Card> NeededTokens
        {
            get
            {
                if (_NeededTokens == null || _NeededTokens.Count == 0)
                {
                    _NeededTokens = GetTokens();
                }
                return _NeededTokens;
            }
        }

        public bool IsDoubleFaced
        {
            get
            {
                int num;
                return !int.TryParse(CardNumber, out num);
            }
        }

        public bool IsToken
        {
            get
            {
                return GetType() == typeof(Token) ? true : false;
            }
        }

        private string _SafeName;
        public virtual string SafeName
        {
            get
            {
                if (string.IsNullOrEmpty(_SafeName))
                {
                    _SafeName = Name.Replace("\"", "");
                    _SafeName = _SafeName.Replace(":", "");
                    _SafeName = _SafeName.Replace("?", "");
                    _SafeName = _SafeName.Replace("ö", "o");
                }
                return _SafeName;
            }
        }

        protected string _MagicAlbumName;
        public virtual string MagicAlbumName
        {
            get
            {
                if (string.IsNullOrEmpty(_MagicAlbumName))
                {
                    _MagicAlbumName = SafeName.Replace(" // ", "_");
                    _MagicAlbumName = _MagicAlbumName.Replace("/", "_");
                    _MagicAlbumName = _MagicAlbumName.Replace("|", "_");
                    _MagicAlbumName = _MagicAlbumName.Replace("Ae", "AE");
                    _MagicAlbumName = _MagicAlbumName.Replace("é", "e");
                    _MagicAlbumName = !string.IsNullOrEmpty(AltArt) ? string.Format("{0} [{1}]", _MagicAlbumName, AltArt) : _MagicAlbumName;
                }
                return _MagicAlbumName;
            }
        }

        protected string _WorstationSafeName;
        public virtual string WorstationSafeName
        {
            get
            {
                if (string.IsNullOrEmpty(_WorstationSafeName))
                {
                    _WorstationSafeName = SafeName.Replace(" // ", "");
                    _WorstationSafeName = _WorstationSafeName.Replace("/", "");
                    _WorstationSafeName = _WorstationSafeName.Replace("|", "");
                    _WorstationSafeName = !string.IsNullOrEmpty(AltArt) ? string.Format("{0}{1}", _WorstationSafeName, AltArt) : _WorstationSafeName;
                }
                return _WorstationSafeName;
            }
        }
        #endregion

        #region Constructor
        public Card(Card card)
        {
            card.CopyType(this);
        }

        public Card(string name, Set set)
        {
            Name = name;
            Set = set;
            IsAltered = false;
        }

        public Card(string name, string set)
        {
            Name = name;
            //Set = new Set(set);
            Set = SetsList.GetSet(set);
            Set = Set == null ? new Set(set) : Set;
            IsAltered = false;
        }
        #endregion

        #region Operator Overload
        public static bool operator ==(Card a, Card b)
        {
            // If both are null, or both are same instance, return true.
            if (Object.ReferenceEquals(a, b))
                return true;

            // If one is null, but not both, return false.
            if (((object)a == null) || ((object)b == null))
                return false;

            // Return true if the fields match:
            return a.ID == b.ID && a.Name == b.Name;
        }

        public static bool operator !=(Card a, Card b)
        {
            return !(a == b);
        }

        public override bool Equals(Object obj)
        {
            return this == obj as Card;
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
        #endregion

        #region Download
        public virtual bool Download(string savefilePath)
        {
            string cardNumber = CardNumber;
            return Download(savefilePath, cardNumber);
        }

        public bool Download(string savefilePath, string cardNumber)
        {
            string URL = String.Format("http://magiccards.info/scans/en/{0}/{1}.jpg", Set.MagicCardsInfo.ToLower(), cardNumber);
            return Download(savefilePath, cardNumber, URL);
        }

        public bool Download(string savefilePath, string cardNumber, string URL)
        {
            if (!string.IsNullOrEmpty(cardNumber))
            {
                bool HasLetter = IsDoubleFaced;

                string CardNumberNoLetters = HasLetter && !IsToken ? Regex.Match(cardNumber, @"\d{1,4}").Value : string.Empty;

                try
                {
                    if (!Directory.Exists(Path.GetDirectoryName(savefilePath)))
                        Directory.CreateDirectory(Path.GetDirectoryName(savefilePath));

                    using (WebClient client = new WebClient())
                    {
                        client.DownloadFile(URL, savefilePath);
                    }
                    return true;
                } catch (Exception)
                {
                    if (HasLetter && !IsToken)
                        return Download(savefilePath, CardNumberNoLetters);

                    return false;
                }
            }

            return false;
        }
        #endregion

        private List<Card> GetTokens()
        {
            List<Card> tokens = new List<Card>();
            Card card = this;

            if (Database.TokensOnly != null)
            {
                foreach (Token token in Database.TokensOnly)
                {
                    if (token.DoesCardNeedMe(card))
                        tokens.Add(token);
                }
            }

            return tokens;
        }

        #region Get Image Path
        public string GetMagicAlbumPath(string BasePath)
        {
            string path;
            //Set.MagicAlbum\ENG\Card.SafeName.jpg
            string LANG = IsToken ? "ENG TOK" : "ENG";
            path = Path.Combine(BasePath, Set.MagicAlbum);
            path = Path.Combine(path, LANG);
            path = Path.Combine(path, MagicAlbumName + ".jpg");
            return path;
        }

        public string GetWorkstationPath(string BasePath)
        {
            string path;
            //WorkstationID\Card.SafeName.full.jpg
            string ID = IsToken ? "TK" : Set.WorkstationID;
            path = Path.Combine(BasePath, ID);
            path = Path.Combine(path, WorstationSafeName + ".full.jpg");
            return path;
        }
        #endregion
    }
}
