﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using System.Threading;
using Ionic.Zip;

namespace MTGRenamer
{
    class Renamer
    {
        private string Source;
        private string Destination;

        public event EventHandler<ProgressEventArgs> Progress;

        private void OnProgress(string Message)
        {
            OnProgress(0, 0, Message);
        }

        private void OnProgress(int CurrentFile, int Maximum, string Message)
        {
            if (Progress != null)
            {
                Progress(this, new ProgressEventArgs(CurrentFile, Maximum, Message));
            }
        }

        public Renamer(string source, string destination)
        {
            Source = source;
            Destination = destination;
        }

        bool Extracted = false;
        List<string> ExtractedPath = new List<string>();
        private void ExtractZip(string source)
        {
            if (File.Exists(source))
            {
                using (ZipFile zip = new ZipFile(source))
                {
                    OnProgress(string.Format("Extracting \"{0}\" ", source));
                    zip.ExtractAll(Path.GetDirectoryName(source), ExtractExistingFileAction.OverwriteSilently);
                    Extracted = true;
                }
                ExtractedPath.Add(Path.GetDirectoryName(source));
            }
        }

        string TempFolders;
        private string GetLocation(Card item, Utils.LocationType LocType, string BasePath)
        {
            string path = string.Empty;
            TempFolders = string.Empty;
            switch (LocType)
            {
                case Utils.LocationType.MagicAlbum:
                    if (item.IsAltered)
                        return GetLocation(item.OriginalCard, LocType, BasePath);

                    path = item.GetMagicAlbumPath(BasePath);
                    break;
                case Utils.LocationType.Wagic:
                    path = Path.Combine(BasePath, item.Set.WagicID);
                    TempFolders = path;
                    path = Path.Combine(path, item.ID + ".jpg");//Save The Tokens by The ID of the cards that needs it

                    //Save The token by it's name only (less file but more potential problems)
                    //path = item.IsToken ? Path.Combine(path, item.Name) : Path.Combine(path, item.ID + ".jpg");
                    break;
                case Utils.LocationType.MagicWorkstation:
                    path = item.GetWorkstationPath(BasePath);
                    break;
                default:
                    break;
            }

            return path;
        }

        private string GetSourceLocation(Card item, Utils.LocationType SourceLocType)
        {
            string source = GetLocation(item, SourceLocType, Source);

            //Extract Files from the zip
            if (SourceLocType == Utils.LocationType.Wagic && !File.Exists(source) && !Extracted)
                ExtractZip(Path.Combine(TempFolders, item.Set.WagicID + ".zip"));

            return source;
        }

        private string GetDestination(Card item, Utils.LocationType DestLocType)
        {
            string destination = GetLocation(item, DestLocType, Destination);

            if (DestLocType == Utils.LocationType.Wagic && !DestinationFoldersToDelete.Contains(TempFolders))
                DestinationFoldersToDelete.Add(TempFolders);

            return destination;
        }

        internal void DoWork(IEnumerable<Card> CardsToProcess, Utils.LocationType SourceLocType, Utils.LocationType DestLocType)
        {
            try
            {
                int CurrentFile = 0;
                int TotalFiles = CardsToProcess.Count();

                Thread t1 = null;

                foreach (Card item in CardsToProcess)
                {
                    CurrentFile++;
                    string source = GetSourceLocation(item, SourceLocType);
                    string destination = GetDestination(item, DestLocType);

                    if (source == destination)
                        continue;

                    bool FileExist = false;

                    if (File.Exists(source))
                    {
                        FileExist = true;
                        CopyFile(source, destination);

                        OnProgress(CurrentFile, TotalFiles, string.Format("Copying \"{0}\" ({1} / {2}) ", item.MagicAlbumName, CurrentFile, TotalFiles));
                    } else
                    {
                        if (item.Download(source))
                        {
                            FileExist = true;
                            CopyFile(source, destination);

                            OnProgress(CurrentFile, TotalFiles, string.Format("File Downloaded \"{0}\" ({1} / {2}) ", item.MagicAlbumName, CurrentFile, TotalFiles));
                        } else
                        {
                            OnProgress(CurrentFile, TotalFiles, string.Format("Not Found \"{0}\" ({1} / {2}) ", item.MagicAlbumName, CurrentFile, TotalFiles));
                        }
                    }

                    //Create Thumnails for Wagic only
                    if (DestLocType == Utils.LocationType.Wagic && FileExist)
                    {
                        t1 = new Thread(() => CreateThumb(source, destination));
                        t1.Start();
                    }
                }

                if (DestLocType == Utils.LocationType.Wagic)
                {
                    //Wait For Thumbs to be Resuzed before continuing
                    if (t1 != null)
                        t1.Join();

                    ZipFiles();
                }

                if (SourceLocType == Utils.LocationType.Wagic && Extracted)
                {
                    foreach (var item in ExtractedPath)
                    {
                        DeleteOldFiles(item);
                    }
                }

                OnProgress(0, TotalFiles, "Finished");

            } catch (ThreadAbortException)
            {
                if (DestLocType == Utils.LocationType.Wagic)
                {
                    foreach (var item in DestinationFoldersToDelete)
                    {
                        DeleteOldFiles(item);
                    }
                }
            }
        }

        private static void CopyFile(string source, string destination)
        {
            try
            {
                Directory.CreateDirectory(Path.GetDirectoryName(destination));
                File.Copy(source, destination, true);
            } catch (Exception)
            {
            }
        }

        private void DeleteOldFiles(string directory)
        {
            var split = directory.Split(Path.DirectorySeparatorChar);
            var Files = Directory.GetFiles(directory, "*.jpg", SearchOption.AllDirectories);//Find Alljpg in all directories
            OnProgress(string.Format("Deleting Old Files From Directory \"{0}\" ", split[split.Count() - 1]));

            try
            {
                //Delete old files
                foreach (var file in Files)
                {
                    if (File.Exists(file))
                        File.Delete(file);
                }
                foreach (var dir in Directory.GetDirectories(directory))
                {
                    if (Directory.Exists(dir))
                        Directory.Delete(dir);
                }
            } catch (Exception)
            {
            }
        }

        List<string> DestinationFoldersToDelete = new List<string>();
        private void ZipFiles()
        {
            //string[] directories = Directory.GetDirectories(Destination);
            int TotalFiles = DestinationFoldersToDelete.Count;
            OnProgress("Zipping");
            int CurrentFile = 1;

            foreach (string dir in DestinationFoldersToDelete)
            {
                var Files = Directory.GetFiles(dir, "*.jpg", SearchOption.AllDirectories);//Find All jpg in all directories
                string[] name = dir.Split(Path.DirectorySeparatorChar);

                string path = Path.Combine(dir, name[name.Length - 1] + ".zip");

                if (Files.Count() > 0)
                {
                    try
                    {
                        if (File.Exists(path))
                            File.Delete(path);

                        using (ZipFile zip = new ZipFile(path))
                        {
                            zip.CompressionLevel = Ionic.Zlib.CompressionLevel.None;
                            //zip.AddDirectory(item);
                            zip.AddFiles(Directory.GetFiles(dir, "*.jpg"), "");
                            zip.AddFiles(Directory.GetFiles(Path.Combine(dir, "thumbnails"), "*.jpg"), "thumbnails");
                            zip.Save();
                        }
                    } catch (Exception)
                    {
                        OnProgress("Error Deleting or Creating Zip Files");
                        //TODO Delete Temp Zip File
                    }

                    DeleteOldFiles(dir);
                }

                OnProgress(CurrentFile, TotalFiles, "Zipped " + name[name.Length - 1]);
                CurrentFile++;
            }
        }

        private void CreateThumb(string source, string destination)
        {
            string path = Path.GetDirectoryName(destination);
            path = Path.Combine(path, "thumbnails");
            path = Path.Combine(path, Path.GetFileName(destination));

            if (File.Exists(source))
                ResizeImage(Image.FromFile(source), path);
        }

        private void ResizeImage(Image image, string path)
        {
            try
            {
                int Width = 60;
                double ratio = (double)image.Height / (double)image.Width;
                int Height = (int)(ratio * Width);

                Bitmap b = new Bitmap(Width, Height);
                using (Graphics g = Graphics.FromImage(b))
                {
                    g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
                    g.DrawImage(image, 0, 0, Width, Height);
                }

                if (!Directory.Exists(Path.GetDirectoryName(path)))
                    Directory.CreateDirectory(Path.GetDirectoryName(path));

                b.Save(path, ImageFormat.Jpeg);
            } catch (Exception)
            {
            }
        }
    }

    class ProgressEventArgs : EventArgs
    {
        public int CurrentFile { get; set; }
        public int Maximum { get; set; }
        public string Message { get; set; }

        public ProgressEventArgs(int currentfile, int maximum)
            : this(currentfile, maximum, string.Empty)
        {
        }

        public ProgressEventArgs(int currentfile, int maximum, string message)
        {
            CurrentFile = currentfile;
            Maximum = maximum;
            Message = message;
        }
    }
}
