﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.IO;
using System.Net;
using System.Text.RegularExpressions;
using Ionic.Zip;
using FuzzyStrings;
using System.Reflection;
using System.Windows.Forms;

namespace MTGRenamer
{
    public abstract class Deck
    {
        public enum Types
        {
            Apprentice,
            //MagicAlbum,
            Forge,
            MagicWorkstation,
            MTGO,
            Wagic
        }

        #region Properties
        public string FilePath { get; set; }
        public string Name { get; set; }
        public string Description { get; set; }
        public string FileName { get; set; }
        public abstract Deck.Types DeckType { get; }
        protected abstract string DefaultExtension { get; }
        protected virtual string CommentChar
        {
            get
            {
                return @"//";
            }
        }

        private List<DeckCard> _Cards;
        public List<DeckCard> Cards
        {
            get
            {
                if (_Cards == null)
                    LoadDeck();

                return _Cards;
            }
            set
            {
                _Cards = value;
            }
        }

        public int CardsMissing
        {
            get
            {
                return _Cards.Where(x => !x.IsInDB).Sum(x => x.Quantity);
            }
        }
        #endregion

        #region Constructor
        protected Deck(string path)
        {
            FilePath = path;
            FileName = Path.GetFileName(FilePath);
            //LoadDeck();
        }

        protected Deck(Deck source)
        {
            source.CopyType(this);
            SaveFilePath();
        }

        private void SaveFilePath()
        {
            //Change FileName of Deck When Saving to Another Format
            SaveFileDialog dialog = new SaveFileDialog();
            dialog.AddExtension = true;
            dialog.DefaultExt = DefaultExtension;
            dialog.Filter = string.Format("{1} | *.{0}", DefaultExtension, this.GetType().Name);
            dialog.FileName = string.IsNullOrEmpty(Name) ? Path.GetFileNameWithoutExtension(FilePath) : Name;
            dialog.InitialDirectory = Path.GetDirectoryName(FilePath);

            if (dialog.ShowDialog() == DialogResult.OK)
            {
                FilePath = dialog.FileName;
            } else
            {
                FilePath = string.Empty;
            }
        }
        #endregion

        #region Load & Save
        protected abstract void LoadFormat(string line, out string name, out string SetID, out string QtyString, out bool SideBoard);
        protected void LoadDeck()
        {
            Cards = new List<DeckCard>();
            foreach (string Line in File.ReadAllLines(FilePath))
            {
                string line = Line.Trim();

                if (Comments(line))
                    continue;

                int Qty = 1;
                Card card = GetCardByID(line);

                bool SideBoard = false;
                if (card == null)
                {
                    string name, SetID, QtyString;
                    LoadFormat(line, out name, out SetID, out QtyString, out SideBoard);

                    if (name == string.Empty)
                        continue;

                    Qty = int.TryParse(QtyString, out Qty) ? Qty : 1;
                    card = GetCard(name, SetID);
                }

                AddCardsToDeck(Qty, card, SideBoard);
            }

            //Set Name to the filename if not found
            if (string.IsNullOrEmpty(Name))
                Name = Path.GetFileNameWithoutExtension(FilePath);
        }

        protected abstract void SaveFormat(ref StringBuilder sb, DeckCard card);
        public virtual void SaveDeck()
        {
            //Create the text File
            StringBuilder sb = new StringBuilder();
            AddCommentsToDeck(sb);

            foreach (DeckCard card in Cards)
            {
                if (card.IsInDB)
                    SaveFormat(ref sb, card);
            }

            //Write File
            if (!string.IsNullOrEmpty(FilePath))
                File.WriteAllText(FilePath, sb.ToString(), Encoding.ASCII);
        }
        #endregion

        #region Methods
        public static Image GetImage(Card card)
        {
            return card.Image();
        }

        protected Card GetCardByID(string line)
        {
            Regex num = new Regex(@"^\d+$");
            Card card = null;
            if (num.IsMatch(line))
            {
                //Card is a number
                string id = num.Match(line).Value;

                card = (from i in Database.Cards
                        where i.ID == id
                        select i).FirstOrDefault();
            }

            return card;
        }

        protected virtual Card GetCard(string name, string SetID)
        {
            return GetCard(name, SetID, x => x.Set.WorkstationID.ToUpper() == SetID.ToUpper());
        }

        protected Card GetCard(string name, string SetID, Predicate<Card> setChooser)
        {
            Card card;
            name = name.Split('|')[0];
            IEnumerable<Card> query = null;
            if (!string.IsNullOrEmpty(SetID))
                query = from i in Database.Cards
                        where setChooser.Invoke(i) && i.Name.ToLower().StartsWith(name.ToLower())
                        select i;

            if (query == null || query.Count() == 0)
                query = from i in Database.Cards
                        where i.Name.ToLower().StartsWith(name.ToLower())
                        select i;

            //Check if Card is not FOund and assign it a unknown card object
            card = query.LastOrDefault();
            TestCardNotFound(ref card, name, SetID);
            return card;
        }

        protected static void TestCardNotFound(ref Card card, string name, string SetID)
        {
            //Card Not Found
            if (card == null)
            {
                Set Findset = SetsList.Sets.SingleOrDefault(x => x.WagicID.ToUpper() == SetID.ToUpper());
                if (Findset == null)
                    Findset = new Set("Unknown", SetID, "Unknown", SetID);

                card = new Card(name, Findset) { ID = "??????", CardNumber = "0", AltArt=string.Empty, Type="Unknown"};
            }
        }

        protected void AddCardsToDeck(int Qty, Card card, bool SideBoard)
        {
            //Add the Cards to the deck
            if (card != null)
            {
                if (!Cards.Exists(x => x.ID == card.ID && x.IsSideboard == SideBoard))
                {
                    Cards.Add(new DeckCard(card, Qty) { IsSideboard = SideBoard });
                } else
                {
                    //Cards.Single(x => x == card).Quantity += Qty;
                    Cards.Single(x => x.ID == card.ID && x.IsSideboard == SideBoard).Quantity += Qty;
                }
            }
        }

        protected virtual bool Comments(string line)
        {
            //Ignore Comments
            if (line.StartsWith(CommentChar))
            {
                line = line.Substring(CommentChar.Length).Trim();
                string NAME = "NAME:";
                if (line.StartsWith(NAME) && string.IsNullOrEmpty(Name))
                    Name = line.Replace(NAME, string.Empty).Trim();

                string DESC = "DESC:";
                if (line.StartsWith(String.Format(DESC, CommentChar)))
                    Description += line.Replace(DESC, string.Empty).Trim() + "\n";

                return true;
            }

            return false;
        }

        protected virtual void AddCommentsToDeck(StringBuilder sb)
        {
            if (!String.IsNullOrEmpty(Name))
                sb.AppendFormat("{0} NAME: {1}", CommentChar, Name).AppendLine();

            if (!String.IsNullOrEmpty(Description))
                Description.ForEachLines(x => sb.AppendFormat("{0} DESC: {1}", CommentChar, x).AppendLine());
        }

        public Deck ConvertTo(Deck.Types deckType)
        {
            //if (this.DeckType != deckType)
            //{
            switch (deckType)
            {
                case Types.Wagic:
                    return new WagicDeck(this);
                //case Types.MagicAlbum: return new MagicAlbumDeck(this);
                case Types.MagicWorkstation:
                    return new WorkstationDeck(this);
                case Types.Apprentice:
                    return new ApprenticeDeck(this);
                case Types.MTGO:
                    return new MTGODeck(this);
                case Types.Forge:
                    return new ForgeDeck(this);
                default:
                    return this;
            }
            //}

            //return this;
        }
        #endregion
    }
}
