﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace MTGRenamer
{
    partial class frmDeckEditor : Form
    {
        #region Properties
        Deck Deck;
        private DeckList DecksList;
        Deck.Types DeckType;
        private string Path;
        #endregion

        #region Constructors
        public frmDeckEditor()
        {
            InitializeComponent();
            this.Icon = Utils.GetIcon();

            Path = Properties.Settings.Default.DeckDir;
            if (!string.IsNullOrEmpty(Path))
                EnableToolbar();

            dgvDeck.AutoGenerateColumns = false;
            cbInput.ComboBox.DataSource = Enum.GetValues(typeof(Deck.Types));
            cbOutput.ComboBox.DataSource = Enum.GetValues(typeof(Deck.Types));
            //ResetLabels();
        }

        public frmDeckEditor(string path)
            : this()
        {
            Path = path;
        }
        #endregion

        #region Labels
        private void ChangeLabels(string description, string name, string filename, bool Reset=false)
        {
            int qty = Deck == null || Reset ? 0 : Deck.Cards.Sum(x => x.Quantity);
            int unique = Deck == null || Reset ? 0 : dgvDeck.Rows.Count;
            int ErrorCards = Deck == null || Reset ? 0 : Deck.CardsMissing;
            int QtySide = Deck == null || Reset ? 0 : Deck.Cards.Where(x => x.IsSideboard).Sum(x => x.Quantity);
            int QtyMain = qty - QtySide;

            lblQty.Text = string.Format("{0} Cards in Deck ({1} Main/{2} Side)", qty.ToString(), QtyMain.ToString(), QtySide.ToString());
            lblUniqueCards.Text = string.Format("{0} Unique Cards in Deck", unique.ToString());
            lblDescription.Text = string.Format("Description : {0}", description);
            lblName.Text = string.Format("Deck Name : {0}", name);
            lblFilename.Text = string.Format("Filename : {0}", filename);
            lblErrorCard.Text = string.Format("{0} Cards Missing in DB", ErrorCards.ToString());
        }

        private void EnableToolbar()
        {
            LoadDecks();
            tsDeck.Visible = true;
            tsWarn.Visible = false;
        }

        private void ResetLabels()
        {
            ChangeLabels(string.Empty, string.Empty, string.Empty, true);
        }
        #endregion

        #region Load
        internal void LoadDecks()
        {
            //Clear List of Decks
            cbDecks.ComboBox.DataSource = null;
            cbDecks.Items.Clear();

            //Clear Lists of Cards
            Deck = null;
            cardsBindingSource.DataSource = null;

            //Load Lists of Decks and Bind them to the ComboBox
            DecksList = new DeckList(Path, DeckType);
            bindingDeckList.DataSource = DecksList.Decks;
            cbDecks.ComboBox.DataSource = bindingDeckList;
            cbDecks.ComboBox.DisplayMember = "FileName";
        }

        private void LoadCardsList()
        {
            //ResetLabels();
            //dsDeck.Deck.Clear();
            Deck = (Deck)cbDecks.SelectedItem;

            if (Deck != null)
            {
                cardsBindingSource.DataSource = Deck.Cards;

                //Load First Image
                if (Deck.Cards.Count > 0)
                    pbCard.Image = Deck.GetImage(Deck.Cards[0]);

                //Change Labels
                ChangeLabels(Deck.Description, Deck.Name, Deck.FileName);
            }
        }
        #endregion

        #region Form Events

        private void btnPath_Click(object sender, EventArgs e)
        {
            if (folderDecksChooser.ShowDialog() == DialogResult.OK)
            {
                Path = folderDecksChooser.SelectedPath;
                SaveSettings();
                EnableToolbar();
            }
        }

        private void btnReload_Click(object sender, EventArgs e)
        {
            LoadDecks();
        }

        private void dgvDeck_CellClick(object sender, DataGridViewCellEventArgs e)
        {
            if (e.RowIndex >= 0)
            {
                DataGridView grid = (DataGridView)sender;
                Card card = grid.Rows[e.RowIndex].DataBoundItem as Card;

                if (Deck != null && card != null)
                    pbCard.Image = Deck.GetImage(card);
            }
        }

        private void cbDecks_SelectedIndexChanged(object sender, EventArgs e)
        {
            LoadCardsList();
        }

        private void btnPrevious_Click(object sender, EventArgs e)
        {
            if (cbDecks.SelectedIndex > 0)
                cbDecks.SelectedIndex--;
        }

        private void btnNext_Click(object sender, EventArgs e)
        {
            if (cbDecks.SelectedIndex + 1 < cbDecks.Items.Count)
                cbDecks.SelectedIndex++;
        }

        private void btnDelete_Click(object sender, EventArgs e)
        {
            if (Deck != null)
            {
                DecksList.Delete(Deck);
                bindingDeckList.ResetBindings(false);
                LoadCardsList();
                btnNext.Select();
            }
        }

        private void btnEdit_Click(object sender, EventArgs e)
        {
            if (Deck != null)
                Process.Start("notepad.exe", Deck.FilePath);
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            try
            {
                if (Deck != null)
                    Deck.ConvertTo((Deck.Types)cbOutput.SelectedItem).SaveDeck();
            } catch (NotImplementedException)
            {
                MessageBox.Show("Not Implemented Yet\n\n" + ((Deck.Types)cbOutput.SelectedItem).ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void cbInput_SelectedIndexChanged(object sender, EventArgs e)
        {
            ResetLabels();
            DeckType = (Deck.Types)cbInput.SelectedIndex;
            cbOutput.SelectedItem = cbInput.SelectedItem;
            LoadDecks();
        }
        #endregion

        #region Data Grid View Formatting
        private void SetCellError(DataGridViewCell cell, bool Reset = false)
        {
            if (!Reset)
            {
                //Set the Red Color & Error Text if the Set is not in the DB
                cell.ErrorText = "Unknown, Not Found";
                cell.Style.ForeColor = Color.Red;
                cell.Style.Font = new Font(dgvDeck.Font, FontStyle.Bold);
            } else
            {
                cell.ErrorText = string.Empty;
                cell.Style.ForeColor = Color.Black;
                cell.Style.Font = dgvDeck.Font;
            }
        }

        private void dgvDeck_CellFormatting(object sender, DataGridViewCellFormattingEventArgs e)
        {
            DataGridView grid = (DataGridView)sender;
            DataGridViewCell cell = grid.Rows[e.RowIndex].Cells[e.ColumnIndex];
            DeckCard card = grid.Rows[e.RowIndex].DataBoundItem as DeckCard;

            if (card != null)
            {
                //Put Sideboard Cards in Blue
                if (card.IsSideboard)
                    cell.Style.ForeColor = Color.Blue;

                //For the Set Column
                if (e.ColumnIndex == ColSet.DisplayIndex)
                {
                    //Add the Full name to the Tooltip
                    cell.ToolTipText = card.Set.GathererName;

                    //Set the Red Color & Error Text if the Set is not in the DB
                    if (card.Set.GathererName == "Unknown")
                        SetCellError(cell);
                }

                //For the Name column
                if (e.ColumnIndex == ColName.DisplayIndex)
                {
                    //Set the Red Color & Error Text if the Set is not in the DB
                    if (card.ID.Contains("?"))
                        SetCellError(cell);
                }
            }
        }
        #endregion

        #region Settings
        private void SaveSettings()
        {
            Properties.Settings.Default.DeckDir = Path;
            Properties.Settings.Default.Save();
        }
        #endregion

        #region Context Menu
        private void contextMenu_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            if (CardClicked != null)
            {
                Deck.Cards[currentRow] = new DeckCard(e.ClickedItem.Tag as Card, CardClicked.Quantity, CardClicked.IsSideboard);

                //Erase Text Error
                foreach (DataGridViewCell item in dgvDeck.Rows[currentRow].Cells)
                    SetCellError(item, true);

                dgvDeck.Refresh();

                //Update Image if the cell clicked is the same has the one selected
                if (dgvDeck.CurrentCell.RowIndex == currentRow)
                    pbCard.Image = Deck.GetImage(Deck.Cards[currentRow]);
            }
        }

        DeckCard CardClicked;
        int currentRow = 0;
        int currentCol = 2;
        private void dgvDeck_CellMouseDown(object sender, DataGridViewCellMouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right && e.RowIndex >= 0)
            {
                CardClicked = Deck.Cards[e.RowIndex];

                contextMenu.Items.Clear();
                currentRow = e.RowIndex;
                currentCol = e.ColumnIndex;
                List<Card> OtherCards = CardClicked.FindAlternativeCards();
                OtherCards.Reverse();

                int MaxSize = 10;
                int startIndex = 0;
                int count = OtherCards.Count;
                if (OtherCards.Count >= MaxSize)
                {
                    //startIndex = OtherCards.Count - MaxSize + 1;
                    count = MaxSize;
                }

                OtherCards = OtherCards.GetRange(startIndex, count);

                OtherCards.ForEach(x => contextMenu.Items.Add(
                String.Format("{0} - {1}", x.AltName, x.Set.GathererName)
                ));

                IEnumerable<ToolStripItem> contextMenuItems = contextMenu.Items.OfType<ToolStripItem>();
                Utils.ForEachTwoList<Card, ToolStripItem>(OtherCards, contextMenuItems, (x, y) => y.Tag = x);
            }
        }
        #endregion
    }
}
